@@ -11,8 +11,7 @@ if ENV['RUBYOPT'] or defined? Gem
   ENV.delete 'RUBYOPT'
 
   require 'rbconfig'
-  cmd = [RbConfig.ruby, 'setup.rb', *ARGV].compact
-  cmd[1,0] = "--disable-gems"
+  cmd = [RbConfig.ruby, '--disable-gems', 'setup.rb', *ARGV]
 
   exec(*cmd)
 end
@@ -22,7 +21,6 @@ Dir.chdir File.dirname(__FILE__)
 $:.unshift File.expand_path('lib')
 require 'rubygems'
 require 'rubygems/gem_runner'
-require 'rubygems/exceptions'
 
 Gem::CommandManager.instance.register_command :setup
 
@@ -32,8 +30,4 @@ if ENV["GEM_PREV_VER"]
 end
 args.unshift 'setup'
 
-begin
-  Gem::GemRunner.new.run args
-rescue Gem::SystemExitException => e
-  exit e.exit_code
-end
+Gem::GemRunner.new.run args
