@@ -1,5 +1,5 @@
 # frozen_string_literal: true
-require 'rubygems/installer_test_case'
+require_relative 'installer_test_case'
 require 'rubygems/uninstaller'
 
 class TestGemUninstaller < Gem::InstallerTestCase
@@ -54,7 +54,7 @@ class TestGemUninstaller < Gem::InstallerTestCase
       uninstaller.remove_all [@spec]
     end
 
-    refute_path_exists @spec.gem_dir
+    assert_path_not_exist @spec.gem_dir
   end
 
   def test_remove_executables_force_keep
@@ -138,7 +138,7 @@ class TestGemUninstaller < Gem::InstallerTestCase
     Dir.mkdir "#{@gemhome}2"
     uninstaller = Gem::Uninstaller.new nil, :install_dir => "#{@gemhome}2"
 
-    e = assert_raises Gem::GemNotInHomeException do
+    e = assert_raise Gem::GemNotInHomeException do
       use_ui ui do
         uninstaller.remove @spec
       end
@@ -149,11 +149,11 @@ class TestGemUninstaller < Gem::InstallerTestCase
 
     assert_equal expected, e.message
 
-    assert_path_exists @spec.gem_dir
+    assert_path_exist @spec.gem_dir
   end
 
   def test_remove_symlinked_gem_home
-    skip "Symlinks not supported or not enabled" unless symlink_supported?
+    pend "Symlinks not supported or not enabled" unless symlink_supported?
 
     Dir.mktmpdir("gem_home") do |dir|
       symlinked_gem_home = "#{dir}/#{File.basename(@gemhome)}"
@@ -166,7 +166,7 @@ class TestGemUninstaller < Gem::InstallerTestCase
         uninstaller.remove @spec
       end
 
-      refute_path_exists @spec.gem_dir
+      assert_path_not_exist @spec.gem_dir
     end
   end
 
@@ -295,9 +295,16 @@ class TestGemUninstaller < Gem::InstallerTestCase
 
     uninstaller = Gem::Uninstaller.new spec.name, :executables => true
 
-    uninstaller.uninstall
-
-    refute_path_exists spec.gem_dir
+    ui = Gem::MockGemUi.new "1\ny\n"
+    use_ui ui do
+      uninstaller.uninstall
+    end
+    expected = "Successfully uninstalled default-2\n" \
+      "There was both a regular copy and a default copy of default-2. The " \
+      "regular copy was successfully uninstalled, but the default copy " \
+      "was left around because default gems can't be removed.\n"
+    assert_equal expected, ui.output
+    assert_path_not_exist spec.gem_dir
   end
 
   def test_uninstall_extension
@@ -318,18 +325,18 @@ create_makefile '#{@spec.name}'
       installer.install
     end
 
-    assert_path_exists @spec.extension_dir, 'sanity check'
+    assert_path_exist @spec.extension_dir, 'sanity check'
 
     uninstaller = Gem::Uninstaller.new @spec.name, :executables => true
     uninstaller.uninstall
 
-    refute_path_exists @spec.extension_dir
+    assert_path_not_exist @spec.extension_dir
   end
 
   def test_uninstall_nonexistent
     uninstaller = Gem::Uninstaller.new 'bogus', :executables => true
 
-    e = assert_raises Gem::InstallError do
+    e = assert_raise Gem::InstallError do
       uninstaller.uninstall
     end
 
@@ -351,7 +358,7 @@ create_makefile '#{@spec.name}'
 
     ui = Gem::MockGemUi.new "n\n"
 
-    assert_raises Gem::DependencyRemovalException do
+    assert_raise Gem::DependencyRemovalException do
       use_ui ui do
         uninstaller.uninstall
       end
@@ -371,16 +378,16 @@ create_makefile '#{@spec.name}'
     gem_dir = File.join @user_spec.gem_dir
 
     Gem.pre_uninstall do
-      assert_path_exists gem_dir
+      assert_path_exist gem_dir
     end
 
     Gem.post_uninstall do
-      refute_path_exists gem_dir
+      assert_path_not_exist gem_dir
     end
 
     uninstaller.uninstall
 
-    refute_path_exists gem_dir
+    assert_path_not_exist gem_dir
 
     assert_same uninstaller, @pre_uninstall_hook_arg
     assert_same uninstaller, @post_uninstall_hook_arg
@@ -392,7 +399,7 @@ create_makefile '#{@spec.name}'
 
     uninstaller = Gem::Uninstaller.new @spec.name, :executables => true
 
-    e = assert_raises Gem::InstallError do
+    e = assert_raise Gem::InstallError do
       uninstaller.uninstall
     end
 
@@ -546,7 +553,7 @@ create_makefile '#{@spec.name}'
     un = Gem::Uninstaller.new('q', :abort_on_dependent => true)
     ui = Gem::MockGemUi.new("y\n")
 
-    assert_raises Gem::DependencyRemovalException do
+    assert_raise Gem::DependencyRemovalException do
       use_ui ui do
         un.uninstall
       end
@@ -619,7 +626,7 @@ create_makefile '#{@spec.name}'
     end
 
     FileUtils.stub :rm_r, stub_rm_r do
-      assert_raises Gem::UninstallError do
+      assert_raise Gem::UninstallError do
         uninstaller.uninstall
       end
     end
