@@ -136,7 +136,11 @@ class Release
   def initialize(version)
     segments = Gem::Version.new(version).segments
 
+    @level = segments[2] != 0 ? :patch : :minor_or_major
+
     @stable_branch = segments[0, 2].join(".")
+    @base_branch = @level == :minor_or_major ? "master" : @stable_branch
+    @previous_stable_branch = @level == :minor_or_major ? "#{segments[0]}.#{segments[1] - 1}" : @stable_branch
 
     rubygems_version = segments.join(".")
     @rubygems = Rubygems.new(rubygems_version, @stable_branch)
@@ -158,33 +162,17 @@ class Release
   def prepare!
     initial_branch = `git rev-parse --abbrev-ref HEAD`.strip
 
-    system("git", "checkout", "-b", @release_branch, @stable_branch, exception: true)
+    system("git", "checkout", "-b", @release_branch, @base_branch, exception: true)
 
     @bundler.set_relevant_pull_requests_from(unreleased_pull_requests)
     @rubygems.set_relevant_pull_requests_from(unreleased_pull_requests)
 
     begin
-      prs = relevant_unreleased_pull_requests
-
-      if prs.any? && !system("git", "cherry-pick", "-x", "-m", "1", *prs.map(&:merge_commit_sha))
-        warn <<~MSG
-
-          Opening a new shell to fix the cherry-pick errors manually. You can do the following now:
-
-          * Find the PR that caused the merge conflict.
-          * If you'd like to include that PR in the release, tag it with an appropriate label. Then type `Ctrl-D` and rerun the task so that the PR is cherry-picked before and the conflict is fixed.
-          * If you don't want to include that PR in the release, fix conflicts manually, run `git add . && git cherry-pick --continue` once done, and if it succeeds, run `exit 0` to resume the release preparation.
-
-        MSG
-
-        unless system(ENV["SHELL"] || "zsh")
-          system("git", "cherry-pick", "--abort", exception: true)
-          raise "Failed to resolve conflicts, resetting original state"
-        end
-      end
+      cherry_pick_pull_requests if @level == :patch
 
       @bundler.cut_changelog!
       system("git", "commit", "-am", "Changelog for Bundler version #{@bundler.version}", exception: true)
+      bundler_changelog = `git show --no-patch --pretty=format:%h`
 
       @bundler.bump_versions!
       system("rake", "update_locked_bundler", exception: true)
@@ -192,16 +180,49 @@ class Release
 
       @rubygems.cut_changelog!
       system("git", "commit", "-am", "Changelog for Rubygems version #{@rubygems.version}", exception: true)
+      rubygems_changelog = `git show --no-patch --pretty=format:%h`
 
       @rubygems.bump_versions!
       system("git", "commit", "-am", "Bump Rubygems version to #{@rubygems.version}", exception: true)
+
+      return if @level == :minor_or_major
+
+      system("git", "checkout", "-b", "cherry_pick_changelogs", "master", exception: true)
+
+      begin
+        system("git", "cherry-pick", bundler_changelog, rubygems_changelog, exception: true)
+      rescue StandardError
+        system("git", "cherry-pick", "--abort")
+        system("git", "branch", "-D", "cherry_pick_changelogs")
+      end
     rescue StandardError
-      system("git", "checkout", initial_branch, exception: true)
-      system("git", "branch", "-D", @release_branch, exception: true)
+      system("git", "checkout", initial_branch)
+      system("git", "branch", "-D", @release_branch)
       raise
     end
   end
 
+  def cherry_pick_pull_requests
+    prs = relevant_unreleased_pull_requests
+
+    if prs.any? && !system("git", "cherry-pick", "-x", "-m", "1", *prs.map(&:merge_commit_sha))
+      warn <<~MSG
+
+        Opening a new shell to fix the cherry-pick errors manually. You can do the following now:
+
+        * Find the PR that caused the merge conflict.
+        * If you'd like to include that PR in the release, tag it with an appropriate label. Then type `Ctrl-D` and rerun the task so that the PR is cherry-picked before and the conflict is fixed.
+        * If you don't want to include that PR in the release, fix conflicts manually, run `git add . && git cherry-pick --continue` once done, and if it succeeds, run `exit 0` to resume the release preparation.
+
+      MSG
+
+      unless system(ENV["SHELL"] || "zsh")
+        system("git", "cherry-pick", "--abort", exception: true)
+        raise "Failed to resolve conflicts, resetting original state"
+      end
+    end
+  end
+
   def cut_changelog!
     @current_library.cut_changelog_for!(unreleased_pull_requests)
   end
@@ -235,7 +256,7 @@ class Release
   end
 
   def unreleased_pr_ids
-    stable_merge_commit_messages = `git log --format=%s --grep "^Merge pull request #" #{@stable_branch}`.split("\n")
+    stable_merge_commit_messages = `git log --format=%s --grep "^Merge pull request #" #{@previous_stable_branch}`.split("\n")
 
     `git log --oneline --grep "^Merge pull request #" origin/master`.split("\n").map do |l|
       _sha, message = l.split(/\s/, 2)
